# ©2023, ANSYS Inc. Unauthorized use, distribution or duplication is prohibited.

"""Frontend of the results file view."""

from dash_extensions.enrich import html


def layout(project_status_info: dict, uid: str) -> html.Div:
    """Layout of the result files view."""

    return html.Div([])
